@@ -3,7 +3,7 @@ import random
 import time
 from logging import Logger
 from pathlib import Path
-from typing import List, Optional, Type, Union
+from typing import Dict, List, Optional, Type
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -20,19 +20,21 @@ from mohou.dataset import (
     AutoEncoderDatasetConfig,
     AutoRegressiveDataset,
     AutoRegressiveDatasetConfig,
-    WeightPolicy,
 )
 from mohou.default import load_default_image_encoder
 from mohou.encoding_rule import EncodingRule
 from mohou.model import (
     LSTM,
+    PBLSTM,
     AutoEncoder,
     AutoEncoderBase,
     AutoEncoderConfig,
     LSTMConfig,
+    PBLSTMConfig,
 )
 from mohou.model.chimera import Chimera, ChimeraConfig, ChimeraDataset
-from mohou.propagator import Propagator
+from mohou.model.lstm import LSTMBase, LSTMConfigBase
+from mohou.propagator import PropagatorBase
 from mohou.trainer import TrainCache, TrainConfig, train
 from mohou.types import (
     AngleVector,
@@ -96,14 +98,21 @@ def train_autoencoder(
 def train_lstm(
     project_path: Path,
     encoding_rule: EncodingRule,
-    model_config: LSTMConfig,
+    model_config: LSTMConfigBase,
     dataset_config: AutoRegressiveDatasetConfig,
     train_config: TrainConfig,
-    weighting: Optional[Union[WeightPolicy, List[np.ndarray]]] = None,
+    model_type: Type[LSTMBase] = LSTM,
     bundle: Optional[EpisodeBundle] = None,
     warm_start: bool = False,
     context_list: Optional[List[np.ndarray]] = None,
 ):
+    # a dirty assertion TODO: do this by generic typing
+    compat_table: Dict[Type[LSTMBase], Type[LSTMConfigBase]] = {
+        LSTM: LSTMConfig,
+        PBLSTM: PBLSTMConfig,
+    }
+    assert model_type in compat_table
+    assert compat_table[model_type] == type(model_config)
 
     if bundle is None:
         bundle = EpisodeBundle.load(project_path)
@@ -118,7 +127,6 @@ def train_lstm(
         bundle,
         encoding_rule,
         augconfig=dataset_config,
-        weighting=weighting,
         static_context_list=context_list,
     )
 
@@ -127,8 +135,8 @@ def train_lstm(
         tcache = TrainCache.load(project_path, LSTM)
         train(project_path, tcache, dataset, config=train_config)
     else:
-        model = LSTM(model_config)
-        tcache = TrainCache.from_model(model)  # type: ignore[var-annotated]
+        model = model_type(model_config)
+        tcache = TrainCache.from_model(model)  # type: ignore
         train(project_path, tcache, dataset, config=train_config)
 
 
@@ -155,7 +163,7 @@ def visualize_train_histories(project_path: Path):
     plot_dir_path = project_path / "train_history"
     plot_dir_path.mkdir(exist_ok=True)
 
-    all_result_paths = TrainCache.filter_result_paths(project_path, None, None)
+    all_result_paths = TrainCache.filter_result_paths(project_path, None)
     for result_path in all_result_paths:
         tcache = TrainCache.load_from_base_path(result_path)
         image_path = plot_dir_path / (result_path.name + ".png")
@@ -237,10 +245,11 @@ def add_text_to_image(image: ImageBase, text: str, color: str):
     return canvas_to_ndarray(fig)
 
 
-def visualize_lstm_propagation(project_path: Path, propagator: Propagator, n_prop: int):
+def visualize_lstm_propagation(project_path: Path, propagator: PropagatorBase, n_prop: int):
     bundle = EpisodeBundle.load(project_path).get_untouch_bundle()
     save_dir_path = project_path / "lstm_result"
     save_dir_path.mkdir(exist_ok=True)
+    prop_name = propagator.__class__.__name__
 
     image_encoder = load_default_image_encoder(project_path)
 
@@ -325,7 +334,7 @@ def visualize_lstm_propagation(project_path: Path, propagator: Propagator, n_pro
         for ax in axs:
             ax.grid()
 
-        image_path = save_dir_path / "seq-{}{}.png".format(AngleVector.__name__, idx)
+        image_path = save_dir_path / "seq-{}-{}{}.png".format(prop_name, AngleVector.__name__, idx)
         fig.savefig(str(image_path), format="png", dpi=300)
         print("saved to {}".format(image_path))
 
@@ -344,7 +353,7 @@ def visualize_lstm_propagation(project_path: Path, propagator: Propagator, n_pro
 
         images_with_text = fed_images_with_text + pred_images_with_text
 
-        image_path = save_dir_path / "result-image{}.gif".format(idx)
+        image_path = save_dir_path / "seq-{}-rgb{}.gif".format(prop_name, idx)
         assert ImageSequenceClip is not None, "check if your moviepy is properly installed"
         clip = ImageSequenceClip(images_with_text, fps=20)
         clip.write_gif(str(image_path), fps=20)
