@@ -819,7 +819,7 @@ class BundleSpec(TypeShapeTableMixin):
     n_untouch_episode: int
     n_average: int
     type_shape_table: Dict[Type[ElementBase], Tuple[int, ...]]
-    meta_data: Optional[MetaData] = None
+    meta_data: MetaData
 
     def to_dict(self) -> Dict:
         d = asdict(self)
@@ -833,6 +833,25 @@ class BundleSpec(TypeShapeTableMixin):
         }
         return cls(**d)
 
+    @classmethod
+    def load(cls, project_path: Path, postfix: Optional[str] = None) -> "BundleSpec":
+        file_path = cls.file_path(project_path, postfix)
+        with file_path.open(mode="r") as f:
+            d = yaml.safe_load(f)
+        return cls.from_dict(d)
+
+    @staticmethod
+    def file_path(project_path: Path, postfix: Optional[str] = None) -> Path:
+        if postfix is None:
+            return project_path / "bundle_spec.yaml"
+        else:
+            return project_path / "bundle_spec-{}.yaml".format(postfix)
+
+    def dump(self, project_path: Path, postfix: Optional[str] = None) -> None:
+        file_path = self.file_path(project_path, postfix)
+        with file_path.open(mode="w") as f:
+            yaml.dump(self.to_dict(), f, default_flow_style=False, sort_keys=False)
+
 
 _bundle_cache: Dict[Tuple[Path, Optional[str]], "EpisodeBundle"] = {}  # used EpisodeBundle.load
 
@@ -1029,12 +1048,7 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
                 shutil.move(str(bundle_dir_path), str(destination_path))
 
         # extra dump just for debugging (the following info is not requried to load bundle)
-        if postfix is None:
-            spec_file_path = project_path / "bundle_spec.yaml"
-        else:
-            spec_file_path = project_path / "bundle_spec-{}.yaml".format(postfix)
-        with spec_file_path.open(mode="w") as f:
-            yaml.dump(self.spec.to_dict(), f, default_flow_style=False, sort_keys=False)
+        self.spec.dump(project_path, postfix)
 
     def get_untouch_bundle(self) -> "EpisodeBundle":
         """get episode bundle which is not used for training."""
